//
// Created by jedi on 10/30/18.
//

#include <iostream>

//Using SDL and standard IO
#include <SDL2/SDL.h>
#include <cstdio>
#include <buttons.h>

SDL_Window *window = nullptr;
SDL_Event event;
SDL_Renderer *renderer;


void hal_init_screen(int width, int height){
    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
        printf("SDL could not initialize! SDL_Error: %s\n", SDL_GetError());
        exit(1);
    }

    //Create window
    window = SDL_CreateWindow("SDL Tutorial", 0, 0, width, height,
                              SDL_WINDOW_SHOWN); //|SDL_WINDOW_BORDERLESS
    if (window == nullptr) {
        printf( "Window could not be created! SDL_Error: %s\n", SDL_GetError() );
        exit(1);
    }

    renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED);
    if (renderer == nullptr) {
        printf("Renderer could not be created! SDL_Error: %s\n", SDL_GetError());
        exit(1);
    }
}

void hal_draw(int x, int y, int b) {
    for (int i = 0; i < 8 *sizeof(int);i++)
    if (b & (1 << i)) {
        SDL_SetRenderDrawColor(renderer, 0, 255, 255, 255);
        SDL_RenderDrawPoint(renderer, x, y + i);
    }else{
        SDL_SetRenderDrawColor(renderer, 0, 0, 0, 0);
        SDL_RenderDrawPoint(renderer, x, y + i);
    }
}

void hal_clear(){
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 0);
    SDL_RenderClear(renderer);
}


bool quit = false;

void hal_render() {


    while (!quit) {
        render();
        SDL_RenderPresent(renderer);
    }

    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();

}

void hal_poll_input(bool *ptr) {
    if (SDL_PollEvent(&event)) {


        switch (event.type) {
            case SDL_KEYDOWN:
                switch (event.key.keysym.sym) {
                    case SDLK_LEFT:
                        ptr[micromenu::I_LEFT] = true;
                        break;
                    case SDLK_RIGHT:
                        ptr[micromenu::I_RIGHT] = true;
                        break;
                    case SDLK_UP:
                        ptr[micromenu::I_UP] = true;
                        break;
                    case SDLK_DOWN:
                        ptr[micromenu::I_DOWN] = true;
                        break;
                    case 'q':
                    case 0x1b: //ESC
                        quit = true;
                        break;
                }
                break;

            case SDL_KEYUP:
                switch (event.key.keysym.sym) {
                    case SDLK_LEFT:
                        ptr[micromenu::I_LEFT] = false;
                        break;
                    case SDLK_RIGHT:
                        ptr[micromenu::I_RIGHT] = false;
                        break;
                    case SDLK_UP:
                        ptr[micromenu::I_UP] = false;
                        break;
                    case SDLK_DOWN:
                        ptr[micromenu::I_DOWN] = false;
                        break;
                }
                break;

            case SDL_QUIT:
                quit = true;
                break;
        }
    } else {
        SDL_Delay(10);
    }
}


void debug(const char* str){
    std::cout << str << std::flush;
}
